/**
 * Jasmine
 * 29/mag/08
 */
package it.uniroma1.dis.jasmine;

import it.uniroma1.dis.jasmine.ui.utils.NumberField;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

/**
 * Grafo di tutti i metodi all'interno di una classe.
 */
public class GraphWrapper {
	private HashMap<String, Graph> graphMap;
	private LinkedList<String> methodList;
	private Graph currentGraph;
	private String currentMethod;
	private String className;
	
	/**
	 * 
	 */
	public GraphWrapper() {
		graphMap = new HashMap<String, Graph>();
		methodList = new LinkedList<String>();
	}
	
	/**
	 * @return the currentMethod
	 */
	public String getCurrentMethod() {
		return currentMethod;
	}
	
	/**
	 * @param currentMethod the currentMethod to set
	 */
	public void setCurrentMethod(String methodName) {
		if (methodName == null)
			return;
		
		if (!graphMap.containsKey(methodName)) {
			Graph g = new Graph();
			g.setMethodName(methodName);
			graphMap.put(methodName, g);
		}
		
		if (!methodList.contains(methodName))
			methodList.add(methodName);
		currentMethod = methodName;
		currentGraph = graphMap.get(methodName);
	}
	
	/**
	 * 
	 */
	public void selectFirstMethod() {
		setCurrentMethod(methodList.getFirst());
	}
	
	/**
	 * @return the methodList
	 */
	public List<String> getMethodList() {
		return Collections.unmodifiableList(methodList);
	}
	
	/**
	 * @return
	 */
	public Collection<Graph> getGraphs() {
		return graphMap.values();
	}
	
	/**
	 * @return the className
	 */
	public String getClassName() {
		return className;
	}

	/**
	 * @param className the className to set
	 */
	public void setClassName(String className) {
		this.className = className;
	}

	/**
	 * @return the nodes
	 */
	public Vector<GraphNode> getNodes() {
		return currentGraph.getNodes();
	}
	
	/**
	 * @param gn
	 * @return
	 */
	public boolean addNode(GraphNode gn) {
		if (gn != null)
			return currentGraph.addNode(gn);
		else
			return false;
	}
	
    /**
     * @return
     */
    public Vector<Variabile> getVariableSet(){
    	return currentGraph.getVariables();
    }
    
    /**
     * @return
     */
    public Vector<ParametroFormale> getParameterSet(){
    	return currentGraph.getParameters();
    }

	/**
	 * @return the loc
	 */
	public int getLoc() {
		return currentGraph.getLoc();
	}

	/**
	 * @param loc the loc to set
	 */
	public void setLoc(int loc) {
		this.currentGraph.setLoc(loc);
	}

	/**
	 * @param v
	 * @return
	 */
	public boolean addVariable(Variabile v) {
		return currentGraph.getVariables().add(v);
	}

	/**
	 * @param pf
	 * @return
	 */
	public boolean addParameter(ParametroFormale pf) {
		return currentGraph.getParameters().add(pf);
	}

	/**
	 * @return the numTermIstr
	 */
	public Vector<Integer> getNumTermIstr() {
		return currentGraph.getNumTermIstr();
	}
	
	/**
	 * @param methodCall the methodCall to set
	 */
	public void setCurrentMethodCall(MethodCall methodCall) {
		this.currentGraph.setMethodCall(methodCall);
	}
	
	/**
	 * Imposta le precondizioni al metodo associato al grafo corrente e a tutti i nodi che chiamano questo metodo.
	 * @param pre
	 */
	public synchronized void setCurrentPreCondition(String pre) {
		this.currentGraph.getMethodCall().setPreCondition(pre);
		for (Graph g : getGraphs()) {
			for (GraphNode n : g.getNodes()) {
				if ((n.getMethodCall() != null) && this.currentGraph.getMethodName().equals((n.getMethodCall().getMethodName())))
					n.getMethodCall().setPreCondition(pre);
			}
		}
	}
	
	/**
	 * Imposta le postcondizioni al metodo associato al grafo corrente e a tutti i nodi che chiamano questo metodo.
	 * @param post
	 */
	public synchronized void setCurrentPostCondition(String post) {
		this.currentGraph.getMethodCall().setPostCondition(post);
		for (Graph g : getGraphs()) {
			for (GraphNode n : g.getNodes()) {
				if ((n.getMethodCall() != null) && this.currentGraph.getMethodName().equals((n.getMethodCall().getMethodName())))
					n.getMethodCall().setPostCondition(post);
			}
		}
	}
	
	/**
	 * @return the variableHash
	 */
	public Hashtable<String, NumberField[]> getCurrentVariableHash() {
		return this.currentGraph.getVariableHash();
	}

	/**
	 * @param variableHash the variableHash to set
	 */
	public void setCurrentVariableHash(Hashtable<String, NumberField[]> variableHash) {
		this.currentGraph.setVariableHash(variableHash);
	}
}
