/**
 * EnterTheMatrix
 * 07/mar/07
 */
package it.uniroma1.dis.jasmine.ui;

import it.uniroma1.dis.jasmine.ui.utils.Settings;
import it.uniroma1.dis.jasmine.ui.utils.TextComponentPopup;
import it.uniroma1.dis.jasmine.ui.utils.Settings.SettingsEnum;

import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 *
 */
public class SettingsDialog extends JDialog implements ActionListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8418065394171323526L;
	
	private JTextField nusmvText;
	private JTextField parametersText;
	private JTextField lastJavaText;
	private JButton findBut;
	private JButton saveBut;
	private JButton cancelBut;
	private JButton delLastBut;
	private JFileChooser chooser;
	
	/**
	 * @param parent
	 */
	public SettingsDialog(JFrame parent) {
		super(parent, true);
		
		setTitle("Impostazioni Jasmine");
		
		getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		
		chooser = new JFileChooser();
		
		JPanel p1 = new JPanel();
		p1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Selezionare l'applicazione NuSMV"));
		nusmvText = new JTextField(20);
		nusmvText.setComponentPopupMenu(new TextComponentPopup(nusmvText));
		nusmvText.setText(Settings.getInstance().getSetting(SettingsEnum.nusmvLocation));
		findBut = new JButton("Sfoglia...");
		findBut.addActionListener(this);
		p1.add(nusmvText);
		p1.add(findBut);
		
		JPanel p2 = new JPanel();
		p2.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Parametri di esecuzione di NuSMV"));
		parametersText = new JTextField(30);
		parametersText.setComponentPopupMenu(new TextComponentPopup(parametersText));
		parametersText.setText(Settings.getInstance().getSetting(SettingsEnum.nusmvParameters));
		p2.add(parametersText);
		
		JPanel p3 = new JPanel();
		p3.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Ultimo file Java aperto"));
		lastJavaText = new JTextField(25);
		lastJavaText.setComponentPopupMenu(new TextComponentPopup(lastJavaText));
		lastJavaText.setEditable(false);
		lastJavaText.setText(Settings.getInstance().getSetting(SettingsEnum.lastJavaFile));
		delLastBut = new JButton("X");
		delLastBut.setToolTipText("Cancella history");
		delLastBut.addActionListener(this);
		p3.add(lastJavaText);
		p3.add(delLastBut);
		
		JPanel p4 = new JPanel();
		saveBut = new JButton("Salva");
		saveBut.addActionListener(this);
		cancelBut = new JButton("Annulla");
		cancelBut.addActionListener(this);
		p4.add(saveBut);
		p4.add(cancelBut);
		
		getContentPane().add(p1);
		getContentPane().add(p2);
		getContentPane().add(p3);
		getContentPane().add(Box.createVerticalStrut(10));
		getContentPane().add(p4);
		
		if ((nusmvText.getText() != null) && (nusmvText.getText().length() > 0))
			chooser.setSelectedFile(new File(nusmvText.getText()));
		
		pack();
		setMinimumSize(getSize());
		
		setLocation((Toolkit.getDefaultToolkit().getScreenSize().width - getSize().width) / 2,
				(Toolkit.getDefaultToolkit().getScreenSize().height - getSize().height) /2);
	}
	
	/**
	 * @param ae
	 */
	public void actionPerformed(ActionEvent ae) {
		if (ae.getSource() == findBut) {
			if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
				File selectedFile = chooser.getSelectedFile();
				if ((selectedFile != null) && selectedFile.exists() && selectedFile.isFile())
					nusmvText.setText(selectedFile.getPath());
				else
					JOptionPane.showMessageDialog(this, "Scegli il percorso completo dell'eseguibile NuSMV", "Scegli un file", JOptionPane.ERROR_MESSAGE);
			}
		} else if (ae.getSource() == saveBut) {
			Settings.getInstance().setSetting(SettingsEnum.nusmvLocation, nusmvText.getText());
			Settings.getInstance().setSetting(SettingsEnum.nusmvParameters, parametersText.getText());
			Settings.getInstance().setSetting(SettingsEnum.lastJavaFile, lastJavaText.getText());
			
			if (Settings.getInstance().storeSettings())
				JOptionPane.showMessageDialog(this, "Salvataggio eseguito.", "Salvataggio", JOptionPane.INFORMATION_MESSAGE);
			else
				JOptionPane.showMessageDialog(this, "Salvataggio non eseguito.", "Salvataggio", JOptionPane.ERROR_MESSAGE);
			
			dispose();
		} else if (ae.getSource() == delLastBut) {
			lastJavaText.setText("");
		} else if (ae.getSource() == cancelBut) {
			dispose();
		}
	}
}
