package it.uniroma1.dis.jasmine.ui.utils;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

/**
 * @author Constantin
 */
public class NumberField extends JTextField {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8246361793597365802L;
	
	/**
	 * 
	 */
	public NumberField() {
		this(null);
	}
	
	/**
	 * @param cols
	 */
	public NumberField(int cols) {
		super(cols);
		addFocusListener(new SelectFocusListener(this));
	}
	
	/**
	 * @param text
	 */
	public NumberField(String text) {
		this.setText(text);
		addFocusListener(new SelectFocusListener(this));
	}

	/**
	 * @return
	 */
	protected Document createDefaultModel() {
		return new NumberDocument();
	}

	/**
	 * @author Constantin
	 */
	static class NumberDocument extends PlainDocument {
		/**
		 * 
		 */
		private static final long serialVersionUID = 6368310251123573433L;
		
		/**
		 * @param offs
		 * @param str
		 * @param a
		 * @throws BadLocationException
		 */
		public void insertString(int offs, String str, AttributeSet a) throws BadLocationException {
			if (str == null)
				return;
			
			StringBuilder sb = new StringBuilder();
			char c;
			for (int i = 0; i < str.length(); i++) {
				c = str.charAt(i);
				if ((c == '-') || (c == ' ') || Character.isDigit(c))
					sb.append(c);
			}
			
			super.insertString(offs, sb.toString(), a);
		}
	}
	
	/**
	 * @author Constantin
	 */
	static class SelectFocusListener implements FocusListener {
		private final NumberField target;
		
		/**
		 * @param target
		 */
		SelectFocusListener(NumberField target) {
			this.target = target;
		}
		
		/*
		 * @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
		 */
		public void focusGained(FocusEvent e) {
			if (e.getSource() == target) {
				target.select(0, target.getText().length());
			}
		}

		/*
		 * @see java.awt.event.FocusListener#focusLost(java.awt.event.FocusEvent)
		 */
		public void focusLost(FocusEvent e) {
		}
	}
}

